package board;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.ArrayList;

import javax.swing.JPanel;

import gameControl.GameState;

@SuppressWarnings("serial")
public class Board extends JPanel {
	private ArrayList<DraughtPiece> pieceList;
	private ArrayList<CandidateSquare> candidateSquareList;
	private ArrayList<DraughtPiece> legalPieces;
	private int originX;
	private int originY;
	
	
	public Board() {
		pieceList=new ArrayList<DraughtPiece>();
		candidateSquareList=new ArrayList<CandidateSquare>();
		legalPieces=new ArrayList<DraughtPiece>();
		
		originX=195;
		originY=30;
		for(int row=1; row<9; ++row) {
			for(int col=1; col<9; ++col) {
				if(row==1 || row==3) {
					if(col%2!=0) {
						pieceList.add(new DraughtPiece(Color.RED, "man", col, row));
					}
				}
				if(row==2) {
					if(col%2==0) {
						pieceList.add(new DraughtPiece(Color.RED, "man", col, row));
					}
				}
				
				if(row==6 || row==8) {
					if(col%2==0) {
						pieceList.add(new DraughtPiece(Color.WHITE, "man", col, row));
					}
				}
				if(row==7) {
					if(col%2!=0) {
						pieceList.add(new DraughtPiece(Color.WHITE, "man", col, row));
					}
				}
			}
		}
	}
	
	
	
	public void paintComponent(Graphics graphicObject) {
		super.paintComponent(graphicObject);
		
		//Loop that draws board
		for(int row=1; row<9; ++row) {
			for(int col=1; col<9; ++col) {
				if(row%2!=0) {
					if(col%2!=0) {
						graphicObject.setColor(Color.BLACK);
					}
					if(col%2==0) {
						graphicObject.setColor(Color.WHITE);
					}
				}
				if(row%2==0) {
					if(col%2!=0) {
						graphicObject.setColor(Color.WHITE);
					}
					if(col%2==0) {
						graphicObject.setColor(Color.BLACK);
					}
				}
				
				graphicObject.fillRect(originX+(62*col), originY+(62*row), 62, 62);
			}
		}
		//End loop that draws board
		
		for(DraughtPiece piece:pieceList) {
			piece.drawDraughtPiece(graphicObject, originX, originY);
		}
		
		
		for(CandidateSquare candidateSquare:candidateSquareList) {
			candidateSquare.draw(graphicObject);		
		}
		
		for(DraughtPiece legalPiece:legalPieces) {
			legalPiece.drawLegalBorder(graphicObject, originX, originY);
		}
		
		
			
		
		
		candidateSquareList.clear();
		
	}
	
	public void repaintBoard(ArrayList<int[]> legalMoves, ArrayList<DraughtPiece> currentState, ArrayList<DraughtPiece> legalPieces) {
		if(legalMoves!=null) {
			for(int[] move:legalMoves) {
				CandidateSquare candidateSquare=new CandidateSquare(move[0], move[1], originX, originY);
				candidateSquareList.add(candidateSquare);
			}
		}
		if(legalMoves==null) {
			candidateSquareList.clear();
		}
		if(currentState!=null) {
			pieceList=currentState;
		}
		if(legalPieces!=null) {
			this.legalPieces=legalPieces;
		}
		this.repaint();
	}
/*	
	public void setPieceList(ArrayList<DraughtPiece> newPieceList) {
		this.pieceList=newPieceList;
	}
	*/
	public ArrayList<DraughtPiece> getPieceList() {
		return pieceList;
	} 
	
	
}
