package gameControl;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.Scanner;

import javax.swing.BoxLayout;
import javax.swing.JFrame;
import javax.swing.JPanel;

import board.Board;
import board.DraughtPiece;
import genetic.GeneticAlgorithm;
import genetic.GeneticPlayer;
import players.BestFirstPlayer;
import players.HumanPlayer;
import players.MiniMaxPlayer;
import players.RandomPlayer;

@SuppressWarnings("serial")
public class EnglishDraughts extends JFrame {

	public EnglishDraughts(String[] args) {
		
		if(args[0].equals("normalGame")) { 
						
			setTitle("English Draughts");
			setSize(new Dimension(1000,1000));
			setLocationRelativeTo(null);
			setDefaultCloseOperation(EXIT_ON_CLOSE);
			
			JPanel contentPanel=new JPanel();
			Board board=new Board();
			
			GameState gameState=new GameState(board.getPieceList());
			
			Scanner scanner=new Scanner(new InputStreamReader(System.in));
			System.out.println("Specify opponent type- h = Human, r = random, b = best first, m = minimax");
			String player2Type=scanner.nextLine();
			HumanPlayer player1=new HumanPlayer(board, 1, gameState);
			
			if(player2Type.equals("h")) {
				HumanPlayer player2=new HumanPlayer(board, 2, gameState);
				player1.setOpponent(player2);
				player2.setOpponent(player1);
			}
			if(player2Type.equals("r")) {
				RandomPlayer player2=new RandomPlayer(board, 2, gameState);
				player1.setOpponent(player2);
				player2.setOpponent(player1);
			}
			if(player2Type.equals("b")) {
				int[] weights=new int[] {100, 200, 50};
				BestFirstPlayer player2=new BestFirstPlayer(board, 2, gameState, weights);
				player1.setOpponent(player2);
				player2.setOpponent(player1);
			}
			if(player2Type.equals("m")) {
				int[] weights=new int[] {100, 200, 50};
				MiniMaxPlayer player2=new MiniMaxPlayer(board, 2, gameState, weights);
				player1.setOpponent(player2);
				player2.setOpponent(player1);
			}

			
					
			contentPanel.setLayout(new BoxLayout(contentPanel, BoxLayout.PAGE_AXIS));
			
			contentPanel.add(board, BorderLayout.NORTH);
			add(contentPanel);
			setVisible(true);
			
			player1.start(); 
		}
		
		if(args[0].equals("runGA")) {
			new GeneticAlgorithm();
		} 
		
		if(args[0].equals("testPlayers")) {
			int totalGames=1000;
			int geneticWeightsWins=0;
			for(int i=0; i<totalGames; ++i) {
				
				ArrayList<DraughtPiece> finalGameState=simulateDraughts();
				
				DraughtPiece testPiece=finalGameState.get(0);
				if(testPiece.getColor()==Color.RED) {
					//Player 2 will always use the genetic weights, and player 2 is always red.
					++geneticWeightsWins;
				}		
			}
			
			System.out.println(geneticWeightsWins+" "+totalGames);
			
		}
		//420 150 107 368 188 363
		/*490 302 410 466 369 303 0 
166 222 355 211 112 444 0 
246 12 45 186 58 467 0 
101 228 231 227 129 229 0 */

}

	

	private ArrayList<DraughtPiece> simulateDraughts() {
		
	
		Board board=new Board();
		GameState gameState=new GameState(board.getPieceList());
		
		//Weights for testing genetic players against eachother.
		int[] p1Weights=new int[] {101, 228, 231, 227, 129, 229};
		int[] p2Weights=new int[] {420, 150, 107, 368, 188, 363};
		
		//Weights for testing BestFirst and MiniMax players. 
		int[] weight1=new int[] {100, 200, 50};
		
		/*DEFINE PLAYERS 1 AND 2 HERE*/
		GeneticPlayer player1=new GeneticPlayer(1, gameState, p1Weights, board);
		GeneticPlayer player2=new GeneticPlayer(2, gameState, p2Weights, board);
		
		player1.setOpponent(player2);
		player2.setOpponent(player1);
		
		player1.start();
		return gameState.getGameStatePieces();
	}



	



	public static void main(String[] args) {
		if(args[0].isEmpty()) {
			System.out.println("Please run with valid command line argument");
		} else {
			new EnglishDraughts(args);
		}
	}

}
