package genetic;

import java.awt.Color;
import java.awt.Dimension;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Comparator;
import java.util.Random;

import board.Board;
import board.DraughtPiece;
import gameControl.GameState;
import moveFinders.LegalPieceFinder;

public class GeneticAlgorithm {
	private int[][] genePool;
	private int totalGenerations=50;
	private int totalPlayers=30;
	private int totalMutations=12;
	private int totalGenes=6;
	private int totalColumns=totalGenes+1;
	
	private int[][] bestPlayers=new int[totalGenerations][totalColumns];
	
	public GeneticAlgorithm() {
		initialiseGenePool();
		System.out.print("Initial Gene Pool");
		printGenePool();
		evolvePlayers();
	
	}

	private void initialiseGenePool() {
		genePool=new int[totalPlayers][totalColumns];
		
		Random randomGenerator=new Random();
		for(int row=0; row<totalPlayers; ++row) {
			for(int col=0; col<totalColumns; ++col) {
				if(col==totalColumns-1) {
					genePool[row][col]=0;
				}
				
				else {
					int randInt=randomGenerator.nextInt(500);
					genePool[row][col]=randInt;
				}
				
			}
		}
		
	}

	private void printGenePool() {
		System.out.print("\n");
		for(int row=0; row<totalPlayers; ++row) {
			for(int col=0; col<totalColumns; ++col) {
				System.out.print(genePool[row][col]+" ");
			}
			System.out.print("\n");
		}
		System.out.print("-----------------");
	}

	private void evolvePlayers() {
		for(int generationNum=0; generationNum<totalGenerations; ++generationNum) {
			for(int player1Num=0; player1Num<totalPlayers; ++player1Num) {
				for(int player2Num=player1Num+1; player2Num<totalPlayers; ++player2Num) {
					int[] p1Weights=genePool[player1Num];
					int[] p2Weights=genePool[player2Num];
					
					ArrayList<DraughtPiece> finalGameState=playDraughts(p1Weights, p2Weights);
					
					String winningPlayer=findWinningPlayer(finalGameState);
					
					if(winningPlayer.equals("white")) {
						genePool[player1Num][totalColumns-1]=genePool[player1Num][totalColumns-1]+1;
					}
					
					if(winningPlayer.equals("red")) {
						genePool[player2Num][totalColumns-1]=genePool[player2Num][totalColumns-1]+1;
					}
				}
			}
			System.out.print("\n");
			int generation=generationNum+1;
			System.out.print("Iteration " +generationNum+" Generation "+generation);
			printGenePool();
			int[] fittestGenes=findFittestGenes();
			
			//Crossover the genes of the players to create children.
			crossoverPool();
			//Introduce mutation into those children.
			mutatePool();
			
			bestPlayers[generationNum]=fittestGenes;
			fittestGenes[totalColumns-1]=0;
			genePool[0]=fittestGenes;
		}
		
		printBestPlayers();
		
		
	}

	private void printBestPlayers() {
		for(int generationNum=0; generationNum<totalGenerations; ++generationNum) {
			System.out.println(bestPlayers[generationNum][0]+" "+bestPlayers[generationNum][1]+" "+bestPlayers[generationNum][2]+" "+bestPlayers[generationNum][3]+" "+bestPlayers[generationNum][4]+" "+bestPlayers[generationNum][5]);
		}
	}

	private void crossoverPool() {
		int[][] sortedPool=sortPool(genePool);
		int[][] childPool=new int[totalPlayers][totalColumns];
		
		int[][] parentPool=new int[14][totalColumns];
		int sortedPoolIndex=15;
		for(int row=0; row<parentPool.length; ++row) {
			parentPool[row]=sortedPool[sortedPoolIndex];
			++sortedPoolIndex;
		}
		
		int childIndex=0;
		for(int row=0; row<parentPool.length;) {
			int[] firstParent=parentPool[row], secondParent=parentPool[row+1];

			Random randomGenerator=new Random();
			
			int[] firstChild=new int[totalColumns], secondChild=new int[totalColumns];
			int[] thirdChild=new int[totalColumns], fourthChild=new int[totalColumns];
			for(int geneNum=0; geneNum<totalGenes; ++geneNum) {
				
				int randInt;
				
				randInt=randomGenerator.nextInt(2);
				if(randInt==0) {
					firstChild[geneNum]=firstParent[geneNum];
				} 
				if(randInt==1) {
					firstChild[geneNum]=secondParent[geneNum];
				}
				
				randInt=randomGenerator.nextInt(2);
				if(randInt==0) {
					secondChild[geneNum]=firstParent[geneNum];
				} 
				if(randInt==1) {
					secondChild[geneNum]=secondParent[geneNum];
				}
				
				randInt=randomGenerator.nextInt(2);
				if(randInt==0) {
					thirdChild[geneNum]=firstParent[geneNum];
				} 
				if(randInt==1) {
					thirdChild[geneNum]=secondParent[geneNum];
				}
				
				randInt=randomGenerator.nextInt(2);
				if(randInt==0) {
					fourthChild[geneNum]=firstParent[geneNum];
				} 
				if(randInt==1) {
					fourthChild[geneNum]=secondParent[geneNum];
				}
				
				
			
			}
			childPool[childIndex]=firstChild;
			childPool[childIndex+1]=secondChild;
			childPool[childIndex+2]=thirdChild;
			childPool[childIndex+3]=fourthChild;
			childIndex=childIndex+4;
			
			if(row==parentPool.length-2) {
				int randInt;
				
				for(int geneNum=0; geneNum<totalGenes; ++geneNum) {
					randInt=randomGenerator.nextInt(2);
					if(randInt==0) {
						firstChild[geneNum]=firstParent[geneNum];
					} 
					if(randInt==1) {
						firstChild[geneNum]=secondParent[geneNum];
					}
					
					randInt=randomGenerator.nextInt(2);
					if(randInt==0) {
						secondChild[geneNum]=firstParent[geneNum];
					} 
					if(randInt==1) {
						secondChild[geneNum]=secondParent[geneNum];
					}
				}
				childPool[28]=firstChild;
				childPool[29]=secondChild;
			}
			row=row+2;
		}
		
		
		
		/*System.out.print("\n");
		for(int i=0; i<totalPlayers; ++i) {
			System.out.println(childPool[i][0]+" "+childPool[i][1]+" "+childPool[i][2]+" "+childPool[i][3]);
		} */
		
		genePool=childPool.clone();
	}

	private void mutatePool() {
		for(int i=0; i<totalMutations; ++i) {
			Random randomGenerator=new Random();
			
			//Add 1 to the random int to avoid mutating the player at index 0. This is because this index will 
			//always be used for the best player from the previous generation.
			int randInt=randomGenerator.nextInt(totalPlayers)+1;
			if(randInt==totalPlayers) {
				--randInt;
			}
			int[] chosenPlayer=genePool[randInt];
			
			int chosenGene=randomGenerator.nextInt(totalGenes);
			chosenPlayer[chosenGene]=randomGenerator.nextInt(500);
		}
		
	}

	private int[][] sortPool(int[][] inputPool) {
		
		Arrays.sort(inputPool, new Comparator<int[]>() {

			@Override
			public int compare(int[] array1, int[] array2) {
				int fitness1=array1[totalColumns-1];
				int fitness2=array2[totalColumns-1];
				
				if(fitness1<fitness2) {
					return -1;
				}
				
				if(fitness1>fitness2) {
					return 1;
				}
				
				else {
					return 0;
				}
			} 
		});
		
		return inputPool;
	}

	private ArrayList<DraughtPiece> playDraughts(int[] p1Weights, int[] p2Weights) {
		Board board=new Board();
		GameState gameState=new GameState(board.getPieceList());
		
		GeneticPlayer player1=new GeneticPlayer(1, gameState, p1Weights, board);
		GeneticPlayer player2=new GeneticPlayer(2, gameState, p2Weights, board);
		
		player1.setOpponent(player2);
		player2.setOpponent(player1);
				
		player1.start();
		
		return gameState.getGameStatePieces();
	}

	private String findWinningPlayer(ArrayList<DraughtPiece> finalGameState) {
		String winningPlayer=null;
		int redPieceNum=0, whitePieceNum=0;
		for(DraughtPiece piece:finalGameState) {
			if(piece.getColor().equals(Color.WHITE)) {
				++whitePieceNum;
			}
			if(piece.getColor().equals(Color.RED)) {
				++redPieceNum;
			}
		}
		
		if(whitePieceNum!=0 && redPieceNum!=0) {
			winningPlayer=checkForBlockedPlayer(finalGameState);
		}
		
		if(whitePieceNum!=0 && redPieceNum==0) {
			winningPlayer="white";
		}
		
		if(whitePieceNum==0 && redPieceNum!=0) {
			winningPlayer="red";
		}
		return winningPlayer;
	}

	private String checkForBlockedPlayer(ArrayList<DraughtPiece> finalGameState) {
		String winningPlayer;
		LegalPieceFinder legalPieceFinder=new LegalPieceFinder();
		ArrayList<DraughtPiece> whiteLegalPieces=legalPieceFinder.findLegalPieces(1, finalGameState);
		ArrayList<DraughtPiece> redLegalPieces=legalPieceFinder.findLegalPieces(2, finalGameState);
		
		if(whiteLegalPieces.isEmpty()) {
			winningPlayer="red";
		}
		if(redLegalPieces.isEmpty()) {
			winningPlayer="white";
		} 
		else {
			winningPlayer="draw";
		}
		return winningPlayer;
	}
	
	private int[] findFittestGenes() {
		int bestFitness=0;
		int[] fittestGenes=new int[totalColumns];
		for(int i=0; i<totalPlayers; ++i) {
			if(genePool[i][totalColumns-1]>bestFitness) {
				bestFitness=genePool[i][totalColumns-1];
				fittestGenes=genePool[i].clone();
			}
		}		
		return fittestGenes;
	}

}
