package moveFinders;

import java.awt.Color;
import java.util.ArrayList;
import java.util.Arrays;

import board.DraughtPiece;

public class JumpFinder {

	
	public ArrayList<int[]> findJumpMoves(DraughtPiece inputPiece, ArrayList<int[]> legalMoves, ArrayList<DraughtPiece> gameState) {
		ArrayList<int[]> jumpMoves=new ArrayList<int[]>();
		if(inputPiece.getPieceType()=="man") {
			if(inputPiece.getColor()==Color.WHITE) {
				jumpMoves=findWhiteManJumpMoves(inputPiece, legalMoves, gameState);
			}
			if(inputPiece.getColor()==Color.RED) {
				jumpMoves=findRedManJumpMoves(inputPiece, legalMoves, gameState);
			}
		}
		if(inputPiece.getPieceType()=="king") {
			jumpMoves=findKingJumpMoves(inputPiece, legalMoves, gameState);
		}
		return jumpMoves;
	}
	
	private ArrayList<int[]> findKingJumpMoves(DraughtPiece inputPiece, ArrayList<int[]> legalMoves, ArrayList<DraughtPiece> gameState) {
		ArrayList<int[]> kingJumpMoves=new ArrayList<int[]>();
		
		for(int[] legalMove:legalMoves) {
			if(squareOccupiedByOpponent(legalMove, inputPiece.getColor(), gameState)) {
				if(legalMove[0]<inputPiece.getCol() && legalMove[1]>inputPiece.getRow()) {
					if(jumpSquareUnoccupied(legalMove[0]-1, legalMove[1]+1, gameState)) {
						int[] jumpMove=new int[] {legalMove[0]-1, legalMove[1]+1};
						kingJumpMoves.add(jumpMove);
					}
				}
				if(legalMove[0]>inputPiece.getCol() && legalMove[1]>inputPiece.getRow()) {
					if(jumpSquareUnoccupied(legalMove[0]+1, legalMove[1]+1, gameState)) {
						int[] jumpMove=new int[] {legalMove[0]+1, legalMove[1]+1};
						kingJumpMoves.add(jumpMove);
					}
				}
				if(legalMove[0]<inputPiece.getCol() && legalMove[1]<inputPiece.getRow()) {
					if(jumpSquareUnoccupied(legalMove[0]-1, legalMove[1]-1, gameState)) {
						int[] jumpMove=new int[] {legalMove[0]-1, legalMove[1]-1};
						kingJumpMoves.add(jumpMove);
					}
				}
				if(legalMove[0]>inputPiece.getCol() && legalMove[1]<inputPiece.getRow()) {
					if(jumpSquareUnoccupied(legalMove[0]+1, legalMove[1]-1, gameState)) {
						int[] jumpMove=new int[] {legalMove[0]+1, legalMove[1]-1};
						kingJumpMoves.add(jumpMove);
					}
				}
			}
		}
		return kingJumpMoves;
	}
	private ArrayList<int[]> findRedManJumpMoves(DraughtPiece inputPiece, ArrayList<int[]> legalMoves, ArrayList<DraughtPiece> gameState) {
		ArrayList<int[]> redManJumpMoves=new ArrayList<int[]>();
		for(int[] legalMove:legalMoves) {
			if(squareOccupiedByOpponent(legalMove, inputPiece.getColor(), gameState)) {
				if(legalMove[0]<inputPiece.getCol()) {
					if(jumpSquareUnoccupied(legalMove[0]-1, legalMove[1]+1, gameState)) {
						int[] jumpMove=new int[] {legalMove[0]-1, legalMove[1]+1};
						redManJumpMoves.add(jumpMove);
					}
				}
				if(legalMove[0]>inputPiece.getCol()) {
					if(jumpSquareUnoccupied(legalMove[0]+1, legalMove[1]+1, gameState)) {
						int[] jumpMove=new int[] {legalMove[0]+1, legalMove[1]+1};
						redManJumpMoves.add(jumpMove);
					}
				}
			}
		}
		
		return redManJumpMoves;
	}

	private ArrayList<int[]> findWhiteManJumpMoves(DraughtPiece inputPiece, ArrayList<int[]> legalMoves, ArrayList<DraughtPiece> gameState) {
		ArrayList<int[]> whiteManJumpMoves=new ArrayList<int[]>();
		for(int[] legalMove:legalMoves) {
			if(squareOccupiedByOpponent(legalMove, inputPiece.getColor(), gameState)) {
				if(legalMove[0]<inputPiece.getCol()) {
					if(jumpSquareUnoccupied(legalMove[0]-1, legalMove[1]-1, gameState)) {
						int[] jumpMove=new int[] {legalMove[0]-1, legalMove[1]-1};
						whiteManJumpMoves.add(jumpMove);
					}
				}
				if(legalMove[0]>inputPiece.getCol()) {
					if(jumpSquareUnoccupied(legalMove[0]+1, legalMove[1]-1, gameState)) {
						int[] jumpMove=new int[] {legalMove[0]+1, legalMove[1]-1};
						whiteManJumpMoves.add(jumpMove);
					}
				}
			}
		}
		return whiteManJumpMoves;
	}


	private boolean jumpSquareUnoccupied(int col, int row, ArrayList<DraughtPiece> gameState) {
		boolean squareUnoccupied=true;
		int[] jumpMove=new int[] {col, row};
		for(DraughtPiece piece:gameState) {
			int[] pieceCoords=new int[] {piece.getCol(), piece.getRow()};
			if(Arrays.equals(jumpMove, pieceCoords)) {
				squareUnoccupied=false;
			}
		}
		return squareUnoccupied;
	}

	private boolean squareOccupiedByOpponent(int[] diagonalMove, Color inputPieceColor, ArrayList<DraughtPiece> gameState) {
		boolean squareOccupiedByOpponent=false;
		for(DraughtPiece piece:gameState) {
			int[] pieceCoords=new int[] {piece.getCol(), piece.getRow()};
			if(Arrays.equals(pieceCoords, diagonalMove) && piece.getColor()!=inputPieceColor) {
				squareOccupiedByOpponent=true;
			}
		}
		return squareOccupiedByOpponent;
	}
	

}
