package moveFinders;

import java.awt.Color;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;

import board.DraughtPiece;

public class LegalMoveFinder {
	
	
	public ArrayList<int[]> findJumpMoves(DraughtPiece inputPiece, ArrayList<int[]> legalMoves, ArrayList<DraughtPiece> gameState) {
		ArrayList<int[]> jumpMoves=new ArrayList<int[]>();
		JumpFinder jumpFinder=new JumpFinder();
		jumpMoves=jumpFinder.findJumpMoves(inputPiece, legalMoves, gameState);
		
		return jumpMoves;
	}
	
	public ArrayList<int[]> findDiagonalMoves(DraughtPiece inputPiece, ArrayList<DraughtPiece> currentState) {
		ArrayList<int[]> legalMoves=new ArrayList<int[]>();
		int[] firstCandidateSquare=new int[2];
		int[] secondCandidateSquare=new int[2];
		int[] thirdCandidateSquare=new int[2];
		int[] fourthCandidateSquare=new int[2];
		
		//White man pieces will always be looking in the row directly above their current row for legal moves. 
		if(inputPiece.getColor()==Color.WHITE) {
			firstCandidateSquare[0]=inputPiece.getCol()+1;
			firstCandidateSquare[1]=inputPiece.getRow()-1;
					
			secondCandidateSquare[0]=inputPiece.getCol()-1;
			secondCandidateSquare[1]=inputPiece.getRow()-1;
						
			legalMoves.add(firstCandidateSquare);
			legalMoves.add(secondCandidateSquare);
		}
		
		//Red man pieces will always look in the row directly below them for legal moves. 
		if(inputPiece.getColor()==Color.RED) {
			firstCandidateSquare[0]=inputPiece.getCol()+1;
			firstCandidateSquare[1]=inputPiece.getRow()+1;
				
			secondCandidateSquare[0]=inputPiece.getCol()-1;
			secondCandidateSquare[1]=inputPiece.getRow()+1;
								
			legalMoves.add(firstCandidateSquare);
			legalMoves.add(secondCandidateSquare);
		}
		
		if(inputPiece.getPieceType()=="king") {
			firstCandidateSquare[0]=inputPiece.getCol()+1;
			firstCandidateSquare[1]=inputPiece.getRow()+1;
				
			secondCandidateSquare[0]=inputPiece.getCol()-1;
			secondCandidateSquare[1]=inputPiece.getRow()+1;
			
			thirdCandidateSquare[0]=inputPiece.getCol()+1;
			thirdCandidateSquare[1]=inputPiece.getRow()-1;
					
			fourthCandidateSquare[0]=inputPiece.getCol()-1;
			fourthCandidateSquare[1]=inputPiece.getRow()-1;
								
			legalMoves.add(firstCandidateSquare);
			legalMoves.add(secondCandidateSquare);
			legalMoves.add(thirdCandidateSquare);
			legalMoves.add(fourthCandidateSquare);

		}			
		return legalMoves;
		
	}

	

	public ArrayList<int[]> removeOutOfBoundsSquares(ArrayList<int[]> legalMoves) {
		ArrayList<int[]> tempList=new ArrayList<int[]>();
		tempList.addAll(legalMoves);
		
		for(int[] move:tempList) {
			if(move[0]<1 || move[0]>8) {
				legalMoves.remove(move);
			}
			else if (move[1]<1 || move[1]>8) {
				legalMoves.remove(move);
			}
		}
		return legalMoves;
	}

	public ArrayList<int[]> removeOccupiedSquares(ArrayList<int[]> legalMoves, ArrayList<DraughtPiece> currentState) {
		ArrayList<int[]> tempList=new ArrayList<int[]>();
		for(DraughtPiece piece:currentState) {
			Iterator<int[]> legalSquaresIterator=legalMoves.iterator();
			while(legalSquaresIterator.hasNext()) {
				int[] legalSquare=legalSquaresIterator.next();
				if(piece.getCol()==legalSquare[0] && piece.getRow()==legalSquare[1]) {
					tempList.add(legalSquare);
				}
			}
		}
		
		for(int[] square:tempList) {
			if(legalMoves.contains(square)) {
				legalMoves.remove(square);
			}
		}
		return legalMoves;
	}
}
