package moveFinders;

import java.awt.Color;
import java.util.ArrayList;
import java.util.Iterator;

import board.DraughtPiece;

public class MoveFinder {
	//Class changes an input DraughtPiece instance's row and column values, depending on the chosenMove argument in
	//the movePiece method. Class also finds the piece taken by the moved piece, if a piece has been taken. 
	
	private boolean pieceKinged;

	public DraughtPiece movePiece(DraughtPiece clickedPiece, int[] chosenMove) {
		pieceKinged=false;
		String currentPieceType=clickedPiece.getPieceType();
		
		DraughtPiece movedPiece=new DraughtPiece(clickedPiece.getColor(), clickedPiece.getPieceType(), chosenMove[0], chosenMove[1]);
		movedPiece.updatePieceType();
		
		
		//Must check if the piece has been kinged. If it has, the turn must end. The boolean value of pieceKinged
		//will be checked in the doMove method of the player whose turn it currently is. If the moved piece has been
		//kinged this turn, pieceKinged will now be true, and that player will know to end its turn. 
		String newPieceType=movedPiece.getPieceType();
		if(!newPieceType.equals(currentPieceType)) {
			pieceKinged=true;
		}
		
		
		return movedPiece;
	}

	public DraughtPiece findTakenPiece(DraughtPiece clickedPiece, int[] chosenMove, ArrayList<DraughtPiece> gameState) {
		DraughtPiece takenPiece=null;
		int[] takenPieceCoords=findTakenPieceCoords(clickedPiece, chosenMove);
		
		Iterator<DraughtPiece> pieceIterator=gameState.iterator();
		while(pieceIterator.hasNext()) {
			DraughtPiece piece=(DraughtPiece) pieceIterator.next();
			if(piece.getCol()==takenPieceCoords[0] && piece.getRow()==takenPieceCoords[1]) {
				takenPiece=piece;
				break;
			}
		}
		
		return takenPiece;
	}
	
	private int[] findTakenPieceCoords(DraughtPiece clickedPiece, int[] chosenMove) {
		int[] takenPieceCoords=new int[2];
		if(clickedPiece.getColor().equals(Color.WHITE) && clickedPiece.getPieceType()=="man") {
			if(clickedPiece.getCol()<chosenMove[0]) {
				takenPieceCoords[0]=chosenMove[0]-1;
				takenPieceCoords[1]=chosenMove[1]+1;
			}
			if(clickedPiece.getCol()>chosenMove[0]) {
				takenPieceCoords[0]=chosenMove[0]+1;
				takenPieceCoords[1]=chosenMove[1]+1;
			}
		}
		if(clickedPiece.getColor().equals(Color.RED) && clickedPiece.getPieceType()=="man") {
			if(clickedPiece.getCol()<chosenMove[0]) {
				takenPieceCoords[0]=chosenMove[0]-1;
				takenPieceCoords[1]=chosenMove[1]-1;
			}
			if(clickedPiece.getCol()>chosenMove[0]) {
				takenPieceCoords[0]=chosenMove[0]+1;
				takenPieceCoords[1]=chosenMove[1]-1;
			}
		}
		
		if(clickedPiece.getPieceType()=="king") {
			if(clickedPiece.getCol()>chosenMove[0] && clickedPiece.getRow()>chosenMove[1]) {
				takenPieceCoords[0]=chosenMove[0]+1;
				takenPieceCoords[1]=chosenMove[1]+1;
			}
			if(clickedPiece.getCol()<chosenMove[0] && clickedPiece.getRow()>chosenMove[1]) {
				takenPieceCoords[0]=chosenMove[0]-1;
				takenPieceCoords[1]=chosenMove[1]+1;
			}
			if(clickedPiece.getCol()>chosenMove[0] && clickedPiece.getRow()<chosenMove[1]) {
				takenPieceCoords[0]=chosenMove[0]+1;
				takenPieceCoords[1]=chosenMove[1]-1;
			}
			if(clickedPiece.getCol()<chosenMove[0] && clickedPiece.getRow()<chosenMove[1]) {
				takenPieceCoords[0]=chosenMove[0]-1;
				takenPieceCoords[1]=chosenMove[1]-1;
			}
		}
		
		return takenPieceCoords;
	}

	public boolean pieceTaken(ArrayList<DraughtPiece> oldGameStatePieces, ArrayList<DraughtPiece> newGameStatePieces) {
		boolean pieceTaken=false;
		if(oldGameStatePieces.size()>newGameStatePieces.size()) {
			pieceTaken=true;
		}
		return pieceTaken;
	}

	public boolean pieceKinged() {
		return pieceKinged;
	}
	
	
}
