package players;

import java.util.ArrayList;
import java.util.Random;

import board.Board;
import board.DraughtPiece;
import gameControl.GameState;
import genetic.GeneticPlayer;
import moveFinders.LegalMoveFinder;
import moveFinders.LegalPieceFinder;
import moveFinders.MoveFinder;
import moveFinders.TurnCalculator;

public class RandomPlayer {
	
	private int playerNum;
	private GameState gameState;
	private LegalPieceFinder legalPieceFinder;
	private ArrayList<DraughtPiece> legalPieces;
	private ArrayList<int[]> legalMoves;
	private Board board;
	
	private GeneticPlayer geneticOpponent;
	private MiniMaxPlayer miniMaxOpponent;
	private HumanPlayer humanOpponent;
	private RandomPlayer randomOpponent;
	private String opponentType;
	private BestFirstPlayer bestFirstOpponent;
	private int turnsTaken;

	public RandomPlayer(Board board, int playerNum, GameState gameState) {
		this.playerNum=playerNum;
		this.gameState=gameState;
		this.board=board;
		legalPieceFinder=new LegalPieceFinder();
		turnsTaken=0;
	}

	public void start() {
		++turnsTaken;
		legalPieceFinder=new LegalPieceFinder();
		legalPieces=legalPieceFinder.findLegalPieces(playerNum, gameState.getGameStatePieces());
		
		if(turnsTaken<100) {
			if(!legalPieces.isEmpty()) {
				Random randGenerator=new Random();
				int randIndex=randGenerator.nextInt(legalPieces.size());
				
				DraughtPiece chosenPiece=legalPieces.get(randIndex);
				doTurn(chosenPiece);
			}
		}
		
		
		else {
			
		}
	}
	
	private void doTurn(DraughtPiece chosenPiece) {
		
		TurnCalculator turnCalculator=new TurnCalculator();
		legalMoves=turnCalculator.findLegalMoves(chosenPiece, gameState.getGameStatePieces());
		
		
		Random randGenerator=new Random();
		int randIndex=randGenerator.nextInt(legalMoves.size());
		
		int[] chosenMove=legalMoves.get(randIndex);
		doMove(chosenMove, chosenPiece);
	}
	
	private void doMove(int[] chosenMove, DraughtPiece chosenPiece) {
		MoveFinder moveFinder=new MoveFinder();
		DraughtPiece movedPiece=moveFinder.movePiece(chosenPiece, chosenMove);
		DraughtPiece takenPiece=moveFinder.findTakenPiece(chosenPiece, chosenMove, gameState.getGameStatePieces());
		
		ArrayList<DraughtPiece> oldGameStatePieces=new ArrayList<DraughtPiece>();
		oldGameStatePieces.addAll(gameState.getGameStatePieces());
		
		ArrayList<DraughtPiece> newGameStatePieces=gameState.getGameStatePieces();
		newGameStatePieces.remove(chosenPiece);
		newGameStatePieces.remove(takenPiece);
		newGameStatePieces.add(movedPiece);
		
		gameState.setGameState(newGameStatePieces);
		board.repaintBoard(null, gameState.getGameStatePieces(), null);
		
		if(moveFinder.pieceTaken(oldGameStatePieces, newGameStatePieces)) {
			if(moveFinder.pieceKinged()) {
				end();
			} 
			
			else {
				legalPieces=legalPieceFinder.findPossibleJumpPiece(movedPiece, gameState.getGameStatePieces());
				if(legalPieces.isEmpty()) {
					end();
				} else {
					board.repaintBoard(null, null, legalPieces);
					doTurn(movedPiece);
				}
			}
			
		}
		
		else {
			end();
		}

	
	}
	
	private void end() {
		gameState.setTurn(gameState.getNextTurn());
		if(opponentType=="miniMax") {
			miniMaxOpponent.start();
		} 
		if(opponentType=="random") {
			randomOpponent.start();
		} 
		
		if(opponentType=="human") {
			humanOpponent.start();
		}
		if(opponentType=="genetic") {
			geneticOpponent.start();
		}
		if(opponentType=="bestFirst") {
			bestFirstOpponent.start();
		}
		
	}
	
	public void setOpponent(MiniMaxPlayer player2) {
		miniMaxOpponent=player2;
		opponentType="miniMax";
	}
	
	public void setOpponent(HumanPlayer player2) {
		humanOpponent=player2;
		opponentType="human";
	}
	
	public void setOpponent(RandomPlayer player2) {
		randomOpponent=player2;
		opponentType="random";
	}

	public void setOpponent(GeneticPlayer player2) {
		geneticOpponent=player2;
		opponentType="genetic";
	}

	public void setOpponent(BestFirstPlayer player2) {
		bestFirstOpponent=player2;
		opponentType="bestFirst";
	}
	


}
