package tree;

import java.awt.Color;
import java.util.ArrayList;

import board.DraughtPiece;
import moveFinders.LegalMoveFinder;
import moveFinders.LegalPieceFinder;
import moveFinders.MoveFinder;
import moveFinders.TurnCalculator;

public class ChildNodeFinder {
	//Class finds all the child nodes of the input root node.
	private ArrayList<ArrayList<DraughtPiece>> childStates;
	

	public void doAllMoves(ArrayList<DraughtPiece> rootState, int playerNum) {
		childStates=new ArrayList<ArrayList<DraughtPiece>>();

		LegalPieceFinder legalPieceFinder=new LegalPieceFinder();
		ArrayList<DraughtPiece> legalPieces=legalPieceFinder.findLegalPieces(playerNum, rootState);
		
		for(DraughtPiece piece:legalPieces) {
			TurnCalculator turnCalculator=new TurnCalculator();
			ArrayList<int[]> legalMoves=turnCalculator.findLegalMoves(piece, rootState);
			
			for(int[] legalMove:legalMoves) {
				generateChildState(legalMove, piece, rootState);
			}
		}
	}

	private void generateChildState(int[] legalMove, DraughtPiece piece, ArrayList<DraughtPiece> rootState) {
		MoveFinder moveFinder=new MoveFinder();
		DraughtPiece movedPiece=moveFinder.movePiece(piece, legalMove);
		DraughtPiece takenPiece=moveFinder.findTakenPiece(piece, legalMove, rootState);
	 	
		ArrayList<DraughtPiece> rootStateCopy=new ArrayList<DraughtPiece>();
		rootStateCopy.addAll(rootState);
		
		rootStateCopy.remove(piece);
		rootStateCopy.remove(takenPiece);
		rootStateCopy.add(movedPiece);
		
		if(moveFinder.pieceTaken(rootState, rootStateCopy)) {
			if(moveFinder.pieceKinged()) {
				childStates.add(rootStateCopy);
			}
			
			else {
				LegalPieceFinder legalPieceFinder=new LegalPieceFinder();
				ArrayList<DraughtPiece> legalPieces=legalPieceFinder.findPossibleJumpPiece(movedPiece, rootStateCopy);
				if(legalPieces.isEmpty()) {
					childStates.add(rootStateCopy);
				} else {
					doJumpMove(rootStateCopy, movedPiece);
				}
			}
		
		} 
		
		else {
			childStates.add(rootStateCopy);
		}
		
	}
	
	private void doJumpMove(ArrayList<DraughtPiece> rootStateCopy, DraughtPiece movedPiece) {
		TurnCalculator turnCalculator=new TurnCalculator();
		ArrayList<int[]> legalMoves=turnCalculator.findLegalMoves(movedPiece, rootStateCopy);
		
		for(int[] legalMove:legalMoves) {
			generateChildState(legalMove, movedPiece, rootStateCopy);
		}
	}

	public ArrayList<ArrayList<DraughtPiece>> getChildStates() {
		return childStates;
	}
}
